@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
-def test_show_fixtures_and_test(testdir):
+def test_show_fixtures_and_test(testdir, dummy_yaml_custom_test):
     """ Verifies that fixtures are not executed. """
-    p = testdir.makepyfile(
+    testdir.makepyfile(
         """
         import pytest
         @pytest.fixture
@@ -12,7 +11,7 @@ def test_show_fixtures_and_test(testdir):
     """
     )
 
-    result = testdir.runpytest("--setup-plan", p)
+    result = testdir.runpytest("--setup-plan")
     assert result.ret == 0
 
     result.stdout.fnmatch_lines(
