@@ -1,6 +1,7 @@
 import argparse
 import distutils.spawn
 import os
+import shlex
 import sys
 
 import py
@@ -298,7 +299,11 @@ def test_argcomplete(testdir, monkeypatch):
         # redirect output from argcomplete to stdin and stderr is not trivial
         # http://stackoverflow.com/q/12589419/1307905
         # so we use bash
-        fp.write('COMP_WORDBREAKS="$COMP_WORDBREAKS" python -m pytest 8>&1 9>&2')
+        fp.write(
+            'COMP_WORDBREAKS="$COMP_WORDBREAKS" {} -m pytest 8>&1 9>&2'.format(
+                shlex.quote(sys.executable)
+            )
+        )
     # alternative would be exteneded Testdir.{run(),_run(),popen()} to be able
     # to handle a keyword argument env that replaces os.environ in popen or
     # extends the copy, advantage: could not forget to restore
